function [ccMean,ccVar] = train_digit_classifier(img,lab)
% FORMAT [ccMean,ccVar] = train_digit_classifier(img,lab)
% Fit the class conditional distributions needed to train a Naive
% Bayesian Gaussian classifier
%
% img  - images of handwriting
% lab  - true labels
%
% ccMean - Class conditional means
% ccVar  - Class conditional variances
%
% No manipulation/regularisation of the variance estimates is done here.
%________________________________________________________________________
% Script:   train_digit_classifier
% Purpose:  Train a N.B.C with Gaussian data
% Author:   Tom Nichols
% Date:     29 Jan 2012
%

% Constants (no magic numbers!)
nDigit = 10;
nPixel = size(img,1);

% sanity check, make sure all digits are present
if (length(unique(lab)) ~= 10)
  error('Not all digits represented!!')
end

% Compute class conditional means and variances
ccMean = zeros(nPixel,nDigit);
ccVar  = zeros(nPixel,nDigit);
for i = 1:nDigit
  Idx = find(lab==i);
  ccMean(:,i) = mean(img(:,Idx),2);
  ccVar(:,i)  = var(img(:,Idx),0,2);
end
